// *
// * BlogCodeApp.cpp
// *
// *  Created on: Sep 1, 2018
// *      Author: Timothy Fish
// *      Website: http://www.timothyfish.com
// *

#include <BlogCodeApp.h>
#include <iostream>
#include <windows.h>
#include <windowsx.h>

BlogCodeApp* BlogCodeApp::instance = 0;

BlogCodeApp& BlogCodeApp::getInstance(){

	if(instance == 0){
		instance = new BlogCodeApp();
	}

	return *instance;
}

BlogCodeApp::BlogCodeApp(): running(0), hInst(0) {
}

BlogCodeApp::~BlogCodeApp() {
}

int BlogCodeApp::begin(HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    PSTR    lpCmdLine,
                    int       nCmdShow)
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);

    if(running) return 0; // only run one copy

    MyRegisterClass(hInstance);

    if (!InitInstance (hInstance, nCmdShow))
    {
    	std::cout<<"Start - GetLastError - InitInstance: "<<GetLastError()<<std::endl;
        return FALSE;
    }

    MSG msg;

    while (GetMessage(&msg, nullptr, 0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return (int) msg.wParam;
}

ATOM BlogCodeApp::MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX windowClass; //window class

	windowClass.cbSize              = sizeof(WNDCLASSEX);
	windowClass.style               = CS_HREDRAW | CS_VREDRAW;
	windowClass.lpfnWndProc         = WndProc;
	windowClass.cbClsExtra          = 0;
	windowClass.cbWndExtra          = 0;
	windowClass.hInstance           = hInstance;
	windowClass.hCursor             = LoadCursor(0, IDC_ARROW);
	windowClass.hbrBackground       = GetSysColorBrush(COLOR_WINDOW);
	windowClass.lpszMenuName        = MAKEINTRESOURCE(IDR_MYMENU);
	windowClass.hIcon               = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_MYICON));
	windowClass.hIconSm             = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_MYICON), IMAGE_ICON, 16, 16, 0);
	windowClass.lpszClassName       = className;
    return RegisterClassEx(&windowClass);
}

BOOL BlogCodeApp::InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	hInst = hInstance; // Store instance handle in our global variable
	RECT    windowRect;

	int width = 600;
	int height = 800;

	windowRect.left =(long)0;       //set left value to 0
	windowRect.right =(long)width;  //set right value to requested width
	windowRect.top =(long)0;        //set top value to 0
	windowRect.bottom =(long)height;//set bottom value to requested height

	AdjustWindowRectEx(&windowRect, WS_OVERLAPPEDWINDOW, FALSE, WS_EX_APPWINDOW | WS_EX_WINDOWEDGE);

	HWND hwnd = CreateWindowEx(0, className,  //class name
			appName,       //app name
			(~WS_MAXIMIZEBOX & ~WS_MINIMIZEBOX & WS_OVERLAPPEDWINDOW) | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
			400, 100,                         //x and y coords
			windowRect.right - windowRect.left,
			windowRect.bottom - windowRect.top,//width, height
			0,                 //handle to parent
			0,                 //handle to menu
			hInstance,    //application instance
			0);                //no xtra params

	if (!hwnd)
	{
		std::cout<<"Start - GetLastError - CreateWindowEx: "<<GetLastError()<<std::endl;
		return FALSE;
	}

    CreateWindow(TEXT("edit"), TEXT(""),
	             WS_VISIBLE | WS_CHILD | SS_LEFT | WS_BORDER | ES_MULTILINE | ES_WANTRETURN | ES_AUTOVSCROLL ,
				 10, 10, width-20, (height-30-50)/2,
				 hwnd, (HMENU) IDM_CODE, NULL, NULL);


    CreateWindow(TEXT("edit"), TEXT("Paste code in the edit box above and click Submit to generate HTML to quote it in a blog."),
    	             WS_VISIBLE | WS_CHILD | SS_LEFT | WS_BORDER | ES_MULTILINE | ES_WANTRETURN | ES_AUTOVSCROLL | ES_READONLY ,
					 10, (height-30-50)/2+20, width-20, (height-30-50)/2,
    				 hwnd, (HMENU) IDM_BLOG, NULL, NULL);
    Edit_LimitText(hwnd, 4096);

	CreateWindow(TEXT("button"), TEXT("Submit"),
			WS_VISIBLE | WS_CHILD ,
			width-90, height-55, 80, 25,
			hwnd, (HMENU) IDM_SUBMIT, NULL, NULL);

	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);

	return TRUE;
}

LRESULT CALLBACK BlogCodeApp::WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HINSTANCE hInstance;
    UNREFERENCED_PARAMETER(hInstance);

    switch (message)
    {
	case WM_CREATE:
		hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
		Clear(hwnd);
		break;
	case WM_COMMAND:
	{
		int wmId = LOWORD(wParam);
		switch (wmId)
		{
		case IDM_SUBMIT:
			Submit(hwnd);
			break;
        case ID_SAVE:
        	MessageBox(hwnd,
        			   "This is what happens when you select Save.",
        			   "Save",
					   MB_OK);

        	break;
        case ID_LOAD:
        	MessageBox(hwnd,
        			   "This is what happens when you select Load.",
        			   "Load",
					   MB_OK);
        	break;
        case ID_UNDO:
        	MessageBox(hwnd,
        			   "This is what happens when you select Undo.",
        			   "Undo",
					   MB_OK);
        	break;
        case ID_REDO:
        	MessageBox(hwnd,
        			   "This is what happens when you select Redo.",
        			   "Redo",
					   MB_OK);
        	break;
        case ID_CLEAR:
        	Clear(hwnd);
        	break;
        case ID_ABOUT:
//        	AboutDlg::getInstance().show(true);
        	DialogBox(hInstance, MAKEINTRESOURCE(IDD_ABOUTBOX), hwnd, About);
            break;
        case ID_VWS:
        	ShellExecute(NULL, "Open", "http://www.timothyfish.com/", NULL, NULL, SW_SHOWNORMAL);
        	break;
        case ID_EXIT:
            DestroyWindow(hwnd);
            break;
		default:
			return DefWindowProc(hwnd, message, wParam, lParam);
		}
	}
		break;
    case WM_PAINT:
        {
            PAINTSTRUCT ps;
            HDC hdc = BeginPaint(hwnd, &ps);
            UNREFERENCED_PARAMETER(hdc);
            // TODO: Add any drawing code that uses hdc here...

            EndPaint(hwnd, &ps);
        }
        break;
    case WM_DESTROY:
        PostQuitMessage(0);
        break;
    default:
        return DefWindowProc(hwnd, message, wParam, lParam);
    }
    return 0;
}

VOID BlogCodeApp::Clear(HWND hwnd)
{
	char buffer[] = "";
	char blog_buffer[] = "Paste code in the edit box above and click Submit to generate HTML to quote it in a blog.";
	SetDlgItemText( hwnd, IDM_CODE, buffer );
	SetDlgItemText( hwnd, IDM_BLOG, blog_buffer );
}

VOID BlogCodeApp::Submit(HWND hwnd)
{
		char code_buffer[2048];
	    char blog_buffer[4096];
	    GetDlgItemText( hwnd, IDM_CODE, code_buffer, 2047 );

	    int i = 0;
	    int j = 0;
	    char openingString[] = "<blockquote><span style=\"font-family: &quot;courier new&quot; , &quot;courier&quot; , monospace;\">";
	    strcpy(&blog_buffer[0], openingString);
	    j = strlen(blog_buffer);

	    while(i < 2048 && j < 4096 && code_buffer[i])
	    {
	    	switch(code_buffer[i])
	    	{
	    	case ' ':
	    		strcpy(&blog_buffer[j], "&nbsp;");
	    		j+=strlen("&nbsp;");
	    		break;
	    	case '<':
	    		strcpy(&blog_buffer[j], "&lt;");
	    		j+=strlen("&lt;");
	    		break;
	    	case '>':
	    		strcpy(&blog_buffer[j], "&gt;");
	    		j+=strlen("&gt;");
	    		break;
	    	case '\t':
	    		strcpy(&blog_buffer[j], "&nbsp;&nbsp;&nbsp;&nbsp;");
	    		j+=strlen("&nbsp;&nbsp;&nbsp;&nbsp;");
	    		break;
 	    	default:
	    		blog_buffer[j] = code_buffer[i];
	    		j++;
	    	}

	    	i++;
	    }

	    char closingString[] = "</span></blockquote>";
	    if(j < int(4096-strlen(closingString)))
	    {
	    	strcpy(&blog_buffer[j], closingString);
	    }
	    SetDlgItemText( hwnd, IDM_BLOG, blog_buffer );
}

INT_PTR CALLBACK BlogCodeApp::About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    switch (message)
    {
    case WM_INITDIALOG:
        return (INT_PTR)TRUE;

    case WM_COMMAND:
        if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
        {
            EndDialog(hDlg, LOWORD(wParam));
            return (INT_PTR)TRUE;
        }
        break;
    }
    return (INT_PTR)FALSE;
}
